@@ -6,38 +6,49 @@ import struct
 from uuid import uuid4
 
 class client:
-    def __init__(self, conn, *callback):
+    def __init__(self, conn, *params):
         self.__s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.__s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        self.__s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
         self.__s.connect(tuple(conn.split(":")[i] if i == 0 else int(conn.split(":")[i]) for i in range(2)))
-        key = RSA.generate(2048)
-        encryptor = PKCS1_OAEP.new(key)
-        self.__crypto = {"server": 0, "client": (key, encryptor)}
-        self.__s.sendall(key.publickey().exportKey(format='PEM', passphrase=None, pkcs=1))
+        self.__crypto = {"server": None, "client": None}
+        options = [x for i,x in enumerate(params) if type(x) == dict]
+        if len(options) > 0:
+            options = options[0]
+            if "encrypted" in options.keys():
+                if options["encrypted"]:
+                    key = RSA.generate(2048)
+                    encryptor = PKCS1_OAEP.new(key)
+                    self.__crypto = {"server": None, "client": (key, encryptor)}
+        self.__s.sendall(self.__crypto["client"][0].publickey().exportKey(format='PEM', passphrase=None, pkcs=1) if self.__crypto["client"] else b"0")
         self.__ehandler = {}
         self.__callbacks = {}
         self.id = ""
         def handleincoming():
             data = self.__s.recv(1024)
             self.id = str(data[:36], 'utf-8')
-            serverkey = RSA.importKey(data[36:], passphrase=None)
-            serverencryptor = PKCS1_OAEP.new(serverkey)
-            self.__crypto["server"] = (serverkey, serverencryptor)
-            if(len(callback) == 1):
+            if not len(data) == 36:
+                serverkey = RSA.importKey(data[36:], passphrase=None)
+                serverencryptor = PKCS1_OAEP.new(serverkey)
+                self.__crypto["server"] = (serverkey, serverencryptor)
+            callback = [x for i,x in enumerate(params) if callable(x)]
+            if len(callback) > 0:
                 callback[0](self)
             while True:
-                data = self.__s.recv(1024)
-                try:
-                    decrypted = self.__crypto["client"][1].decrypt(data)
-                    datalist = struct.unpack(*struct.unpack("%ds%ss"%struct.unpack('ii', decrypted[:8]), decrypted[8:]))
-                except Exception:
-                    raise Exception
-                else:
+                def handleData(data):
+                    if self.__crypto["client"]:
+                        data = self.__crypto["client"][1].decrypt(data)
+                    lengths = struct.unpack('ii', data[:8])
+                    if sum(lengths) < len(data[8:]):
+                        handleData(data[8+sum(lengths):])
+                        data = data[:8+sum(lengths)]
+                    datalist = struct.unpack(*struct.unpack("%ds%ss"%lengths, data[8:]))
                     datalist = [str(i, 'utf-8') if type(i) == bytes else i for i in datalist]
                     if type(datalist[0]) == str:
-                        if datalist[0][:15] == "@gtcp:callback:":
+                        if datalist[0][15:] in self.__callbacks.keys():
                             self.__callbacks[datalist[0][15:]](*datalist[1:])
                             del self.__callbacks[datalist[0][15:]]
-                            continue
+                            return
                     if datalist[0] in self.__ehandler.keys():
                         for i in range(len(datalist)):
                             if type(datalist[i]) == str:
@@ -47,6 +58,11 @@ class client:
                                         self.emit(cbeid, *args)
                                     datalist[i] = vcallback
                         self.__ehandler[datalist[0]](*datalist[1:len(datalist)])
+                try:
+                    data = self.__s.recv(1024)
+                    handleData(data)
+                except Exception:
+                    raise Exception
         t = Thread(target=handleincoming)
         t.start()
     def emit(self, *data):
@@ -71,9 +87,8 @@ class client:
                 mdata.append(i)
         packeddata = struct.pack(types, *mdata)
         encodeddata = struct.pack(f'ii{len(types)}s{len(packeddata)}s', len(types), len(packeddata), bytearray(types, 'utf-8'), packeddata)
-        try:
-            self.__s.sendall(self.__crypto["server"][1].encrypt(encodeddata))
-        except TypeError:
-            raise Exception("Server credentials not recieved yet")
+        if self.__crypto["server"]:
+            encodeddata = self.__crypto["server"][1].encrypt(encodeddata)
+        self.__s.sendall(encodeddata)
     def on(self, event, callback):
         self.__ehandler[event] = callback
\ No newline at end of file
